/*
 * Copyright (c) 2002, Regents of the University of California. All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL WEBCOLLAB.COM OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
 package edu.uci.isr.yancees.server.plugin.notification.pull;

/**
 * <p>Title: Yancees Notification Server</p>
 * <p>Description: Yet ANother Configurable Extensible Event Service</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: School of Information and Computer Science - University of California, Irvine</p>
 * @author Roberto Silveira Silva Filho
 * @version 1.0
 */

import java.util.Date;
import java.util.Vector;

import org.w3c.dom.Node;

import edu.uci.isr.yancees.EventInterface;
import edu.uci.isr.yancees.plugin.AbstractMOPlugin;
import edu.uci.isr.yancees.plugin.MOPluginInterface;

public class PullPlugin
      extends AbstractMOPlugin {

   public static final String PULL_TAG = "pull";
   
   boolean print = edu.uci.isr.yancees.YanceesProperties.getInstance().PRINT_DEBUG;
   Vector eventsBuffer;

   /**
    * @param subTree is the DOM tree this plugin is responsible for executing
    * evalutation of this plugin is published.
    */
   public PullPlugin(Node subTree) {
      super(subTree);

      if (print) {
         System.out.println("PullPlugin: new instance...");
      }

      eventsBuffer = new Vector();

   }

   /**
    * Receives a notification from another plug-in
    * @param evt is the event received
    * @param source is the plug-in sending the notification.
    */
   public void receivePluginNotification(EventInterface evt, MOPluginInterface source) {
      if (print) {
         System.out.println("PullPlugin: got a plugin notification...");
      }
      if (print)
         System.out.println("PullPlugin: Bufferising one event...");
      eventsBuffer.add(evt);
     //publishOutput(evt);
   }

   /**
    * Receives a list of events as notifications from another plug-in
    * @param evtList is the list of events received
    * @param source is the plug-in sending the notification.
    **/
   public void receivePluginNotification(EventInterface[] evtList, MOPluginInterface source) {

      if (print) {
         System.out.println(
               "PullPlugin: got plugin notification with multiple events...");
      }

      if (print) {
         System.out.println("PullPlugin: Bufferizing " + evtList.length +
                            " events...");
         System.out.println("PullPlugin: currently have "+eventsBuffer.size()+" events stored.");
      }
      for (int i = 0; i < evtList.length; i++) {
         eventsBuffer.add(evtList[i]);
      }
      //publishOutput(evtList);
   }

   /**
    * This method is supposed to be used by the poll protocol plug-in in order
    * to collect the bufferized events
    * @return the currently stored events and empties the buffer.
    */
   public Vector poll() {
      Vector currentEvents = eventsBuffer;
      eventsBuffer = new Vector();

      if (print) {
         System.out.println("PullPlugin: poll invoked!");
         System.out.println(
               "PullPlugin: emptying buffer and returning events...");
      }

      return currentEvents;
   }

   /**
    * Removes the events that are older than a specified time/date.
    * @param date the date before which all events in the buffer are erased.
    */
   public void removeEventsOlderThan(Date date) {

   }


}