/*
 * Copyright (c) 2002, Regents of the University of California. All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL WEBCOLLAB.COM OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package edu.uci.isr.yancees.server.plugin.protocol.peerpublish;

/**
 * <p>
 * Title: Yancees Notification Server
 * </p>
 * <p>
 * Description: Yet ANother Configurable Extensible Event Service
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Company: School of Information and Computer Science - University of
 * California, Irvine
 * </p>
 * 
 * @author Roberto Silveira Silva Filho
 * @version 1.0
 */

import java.rmi.RemoteException;

import edu.uci.isr.yancees.ArchitectureManager;
import edu.uci.isr.yancees.EventInterface;
import edu.uci.isr.yancees.SubscriberInterface;
import edu.uci.isr.yancees.dispatcher.DispatcherException;
import edu.uci.isr.yancees.dispatcher.EventDispatcherInterface;
import edu.uci.isr.yancees.plugin.RemoteAbstractProtocolPlugin;

public class RemotePeerPublishProtocolPluginImplementation
		extends
			RemoteAbstractProtocolPlugin
		implements
			RemotePeerPublishProtocolPluginInterface {

	
	private boolean print = edu.uci.isr.yancees.YanceesProperties.getInstance().PRINT_DEBUG;
	//private boolean print = true;
	private EventDispatcherInterface dispatcher;

	/**
	 * @param si is the subscriber interface of the plug-in creator
	 */
	public RemotePeerPublishProtocolPluginImplementation(SubscriberInterface si)
			throws RemoteException {
		super(si);
		dispatcher = ArchitectureManager.getInstance().getEventDispatcher();
	}

	/**
	 * Receive events from a peer and propagate them to the local dispathcher.
	 * By bypassing the regular publish() command, we prevent the circular
	 * propagation of the events.
	 * 
	 * @param event is the event to be published.
	 * @throws RemoteException
	 */
	public void publishEventFromPeer(final EventInterface event)
			throws RemoteException {
		
		Thread t = new Thread() {
			public void run() {
				if (print) {
					System.out.println("RemotePeerPublishProtocolPluginImplementation: received event from peer:");
					System.out.println(event.toString());					
				}

				try {
					dispatcher.publish(event);
				} catch (DispatcherException e) {
					System.out
							.println("RemotePeerPublishProtocolPluginImplementation: ");
					e.printStackTrace();
				}

			}
		};
		t.start();
		
	}

	/**
	 * Receive events from a peer and propagate them to the local dispathcher.
	 * By bypassing the regular publish() command, we prevent the circular
	 * propagation of the events.
	 * 
	 * @param eventList
	 *            is the list of events to be published.
	 * @throws RemoteException
	 */
	public void publishEventsFromPeer(final EventInterface[] eventList)
			throws RemoteException {
		
		Thread t = new Thread() {
			public void run() {

				if (print)
					System.out.println("publishEventFromPeer event array called");

				try {
					for (int i = 0; i < eventList.length; i++) {
						dispatcher.publish(eventList[i]);
					}
				} catch (DispatcherException e) {
					System.out
							.println("RemotePeerPublishProtocolPluginImplementation: ");
					e.printStackTrace();
				}

			}
		};
		t.start();
	}

}