/*
 * Copyright (c) 2002, Regents of the University of California. All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL WEBCOLLAB.COM OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
 /*
 * Created on Sep 5, 2003
 *
 * <p>Title: Yancees Notification Server</p>
 * <p>Description: Yet ANother Configurable Extensible Event Service</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: School of Information and Computer Science - University of California, Irvine</p>
 * @author Roberto Silveira Silva Filho [rsilvafi@ics.uci.edu]
 * @version 1.0
 */
package edu.uci.isr.yancees.server.service.cassius;

import java.util.HashMap;

import org.w3c.dom.Node;

/**
 * @author Roberto Silveira Silva Filho [rsilvafi@ics.uci.edu]
 *
 * This class implements the record for:
 * 
 * <xsd:complexType name="ChangeAccountCommandType">
 * 	<xsd:sequence>
 *    	<xsd:element name="accountName" type="xsd:string" minOccurs="1" maxOccurs="1"/>
 *    	<xsd:element name="newAccountName" type="xsd:string" minOccurs="0" maxOccurs="1"/>
 *    	<xsd:element name="newAccountClass" type="xsd:string" minOccurs="0" maxOccurs="1"/>
 *    	<xsd:element name="newDescription" type="xsd:string" minOccurs="0" maxOccurs="1"/>
 * 	</xsd:sequence>
 * </xsd:complexType>
 * 
 */
public class CassAccount {

	private String name;
	private String description;
	
	
	private HashMap objectMap = new HashMap();
		
	/**
	 * Constructor
	 */
	public CassAccount(String accountName, String desc) {
		super();
		description = desc;
		name = accountName;
	}
	
	public String getName() {
		return name;
	}
	
	public String getDescription() {
		return description;
	}
	
	public void setDescription(String newDescription) {
		description = newDescription;
	}
	
	public void addObject(CassObject obj) {
		objectMap.put(obj.getId(), obj);
	}
	
	public void removeObject(CassObject obj) {
		objectMap.remove(obj.getId());
	}
	
	public CassObject getObject(String objId) {
		return (CassObject) objectMap.get(objId);
	}
	
	public void clearObjects() {
		objectMap.clear();
	}
	
	public CassObject[] getObjectList() {
		CassObject[] list = null;
		list = new CassObject[objectMap.size()];
		objectMap.entrySet().toArray(list);
		return list;
	}
	
	public Node getObjectHierarchy() {
		Node hierarchy = null;
		
		//TODO: getObjectHierarchy()
		
		return hierarchy;
	}

}
