/*
 * Copyright (c) 2003, Regents of the University of California. All rights
 * reserved.
 * 
 * =================================================================== 
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if any, must
 * include the following acknowledgment: "This product includes software
 * developed by the Institute for Software Research at University of California,
 * Irvine" Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and "University of
 * California" must not be used to endorse or promote products derived from this
 * software without prior written permission. For written permission, please
 * contact rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called "YANCEES", nor may
 * "YANCEES" appear in their name, without prior written permission of the
 * University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * INSTITUTE FOR SOFTWARE RESEARCH AT THE UNIVERSITY OF CALIFORNIA, IRVINE, OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */

package edu.uci.isr.yancees.server.service.jmdns;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;

import javax.jmdns.JmDNS;

/**
 * This service connects YANCESS with its peers in a subnetwork. It uses the
 * service location protocol (SLP)
 * 
 * This special pocket PC version does not use java.net.NetworkInterface, which
 * is only available at JDK1.4. In order to run on pocketPC, we must use JDK1.3
 * compatible components.
 */
public class JmDNSServicePocketPC extends JmDNSService {

	public JmDNSServicePocketPC() {
		super();
	}
	
	public void initialize() {
		// Pocket PCs, so far, have only one network interface, hence, we will
		// always get the right address by using InetAddress class. Hence, the
		// code here is much simpler than the conde for a regular PC or Mac.

		InetAddress address = null;
		try {
			address = InetAddress.getLocalHost();
		} catch (UnknownHostException e) {
			System.out.println("JmDNSServicePocketPC: could not determine local network address");
			e.printStackTrace();
		}

		JmDNS myJmDNS = null;
		try {
			// reference to the JmDNS runtime, which implements the
			// JmDNS protocol for the provided interface address...
			myJmDNS = new JmDNS(address);
		} catch (IOException e1) {
			System.out
					.println("JmDNSService: ERROR when initializing JmDNS for address: "
							+ address);
			e1.printStackTrace();
		}

		registerItselfAsJMDNSListener(myJmDNS);

		// composes the name that this service will be known in the
		// JmDNS service for that specific address.
		String hostname = address.getHostAddress();
		String myURL = "//" + hostname + "/" + LOOKUP_NAME;
		_myURLList.add(myURL);

		System.out.println("JmDNSService: Registering local copy using URL: "
				+ myURL);

		advertiseThisYanceesInstance(myURL, myJmDNS);
	}

}